/////////////////////////////////////////////////////////////////////////////////////
//    This file is part of Voxel Mages 						  ///
//    Copyright (C) 2015 Vicent Gimenez Alventosa				  ///
//										  ///
//    Voxel Mages is free software: you can redistribute it and/or modify	  ///
//    it under the terms of the GNU General Public License as published by	  ///
//    the Free Software Foundation, either version 3 of the License, or		  ///
//    (at your option) any later version.					  ///
//										  ///
//    Foobar is distributed in the hope that it will be useful,			  ///
//    but WITHOUT ANY WARRANTY; without even the implied warranty of		  ///
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the		  ///
//    GNU General Public License for more details.				  ///
//										  ///
//    You should have received a copy of the GNU General Public License		  ///
//    along with Foobar.  If not, see <http://www.gnu.org/licenses/>.		  ///
/////////////////////////////////////////////////////////////////////////////////////    

//Constructor d'objectes 


#ifndef OBJECTES_H
#define OBJECTES_H

#include<iostream>
#include<cstdlib>
#include<ctime>
#include<cstdio>
#include<fstream>
#include<cmath>
#include<GL/glut.h>
#include<string>
#include<sstream>
#include<fstream>
#include<cmath>
#include<omp.h>		//Permet usar el OpenMP pera paral·lelitzar els calculs

#define PI 3.141592653589793
#define INFINIT 999999999999999999

//TOTES LES DIMENSIONS ESTAN EN cm//

extern float Proporcio_Lletra;
extern int PuntsGenVoxel;		//Determina quants punts generem per voxel a les figures
extern int NumTotalFig;
extern int NumVoxels[3];	//Numero de voxels en cada eix
extern int NumVoxelsAntic[3];		//Copia de num voxels per a poder saber les dimensions de l'antiga matriu de voxels i borrar-la
extern double DimVoxels[3];	//Dimensions dels voxels (x,y,z)
extern float RadiMaxVoxel;
extern float DimEspai[3];	//Dimensions de l'espai
//extern float Voxels[200][200][200][2];		//Ordenem els voxels en numero que ocupa en els eixos x,y,z i el [2] correspon a material i densitat
//extern float VoxelsCopia[200][200][200][2];
extern float**** Voxels;		//Ordenem els voxels en numero que ocupa en els eixos x,y,z i el [2] correspon a material i densitat
extern float**** VoxelsCopia;
extern bool CrearPunts;		//Indica si s'han de crear els punts i omplir els voxels amb aquest objecte
extern int MaxMaterial;		//Indica quin número de material és el més alt
extern int MaxPrioritat;			//Guarda el valor de la prioritat més alta
extern int PrioritatsAsignades[1][1001];	//Guarda les prioritats que s'ha utilitzat
//ES SUPOSA QUE L'EIX DE COORDENADES ÉS L'UTILITZAT EN FÍSICA AMB LES DIRECCIONS POSITIVES: y-->  z ^  x cap a fora
//LES COORDENADES DELS OBJECTES HAN D'ESTAR RESPECTE A L'ORIGEN DE COORDENADES 
void ConstruirMatriuVoxels(int PixelsX,int PixelsY,int PixelsZ);
class Objectes
{
protected:
  
  //FUNCIONS//
  void CrearLlista();  //Crea la llista on es guarda la funció dibuixar ja feta, aixi facilita el calcul.
  void AsignarNumero();	//Asigna un numero a la figura
  void OmplirPuntsInt();		//Omplira el vector amb els punts d'interacció
  
  
public:

  //VARIABLES//
  float radiMax;
  float Densitat;
  GLuint llista;	//El numero de la llista on s'emmagatzemen els vertex
  bool llistaCreada;	//Ens diu si la llista s'ha creat. La llista ha de crear-se despres de la funcio iniciar del programa 
  GLfloat ColorRGBA[4]; //Color de l'objecte
  GLfloat ColorEspecular[4]; //Color que es reflexa
  GLfloat Shininess;	//Concentració de la reflexio de la llum
  int NFigura;			//asigna a cada objecte un nombre	
  
  
  //VARIABLES//
  int PDins[100];	//Guarda les prioritats que es pinten dins l'objecte
  double Alfa;		//Angles de Euler
  double Beta;
  double Gamma;	
  double Centre[3];	//Centre de l'objecte
  int Material;		//Indica quin material és 
  int Prioritat;	//Indica la prioritat en que es pinta
  //FUNCIONS//
  Objectes();
  int  NumeroFigura() const;			//retorna el número de figura
  void CanviarMaterial(int NouMaterial,float nouDensitat);
  void CanviarColor(GLfloat nouColorR,GLfloat nouColorG,GLfloat nouColorB);
  void CanviarOpacitat(GLfloat);
  void CanviarCentre(double nouCentreX,double nouCentreY,double nouCentreZ);
  void Girar(double nouAlfa, double nouBeta ,double nouGamma);
  float LlegirDensitat() const;
  virtual void Dibuixar() {}; //Aquesta funcio virtual servira per a que crearLlista puga accedir a dibuixar.
  void Pintar();		//Aquesta funcio pintara usant la llista de forma més rapida i carregant de menys calculs
  ~Objectes(){};
};

class Rectangle : public Objectes
{
private:
  public:
  double Dimensions[3]; //Dimensions del rectangle {x,y,z}

    Rectangle();
  void CanviarDimensions(double nouDimensionsX,double nouDimensionsY,double nouDimensionsZ);	//funcio per a canviar les dimensions del rectangle
  void Dibuixar();		//Funcio que dibuixa el rectangle
  Rectangle operator=(const Rectangle &c);		//Permet igualar una variable rectangle a un altra fent una copia
  ~Rectangle(){};		//Destructor
};

class Cilindre : public Objectes
{
private:
public:
  bool Tub;      //Controla si es tracta d'un tub buit, el radi interior del qual ve determinat pel tercer parametre de la variable "Dimensions" i el exterior pel segon parametre
  double Dimensions[3];//Dimensions del cilindre {altura,radi superior,radi inferior}
  
  Cilindre();
  Cilindre operator=(const Cilindre &c);
  void CanviarDimensions(double nouDimensionsA,double nouDimensionsRS,double nouDimensionsRI);	//funcio per a canviar les dimensions del cilindre
  void ActivarTub(bool Activar);
  void Dibuixar();
  ~Cilindre(){};
};

class Esfera : public Objectes
{
private:
public:
  double Dimensions; //Dimensions de la esfera (radi)
  
  Esfera();
  Esfera operator=(const Esfera &c);
  void CanviarDimensions(double nouDimensions);	//funcio per a canviar les dimensions de la esfera
  void Dibuixar();
  ~Esfera(){};
};

class Pla : public Objectes
{
public:
  double Direccio[3];//Direccio on apunta el pla
  double Dimensions; //Radi del cercle que representa el pla
  
  Pla();
  Pla operator=(const Pla &c);
  void CanviarDireccio(double nouDireccioX,double nouDireccioY,double nouDireccioZ);	//funcio per a canviar les dimensions de la esfera
  void CanviarDimensions(double nouDimensions);
  void Dibuixar();
  ~Pla(){};
};

class SeccioEsfera : public Objectes
{
public:
  double Direccio[3];//Direccio on apunta el pla
  double Dimensions[2]; //{Radi de l'esfera , altura de tall}
  
  SeccioEsfera();
  SeccioEsfera operator=(const SeccioEsfera &c);
  void CanviarDireccio(double nouDireccioX,double nouDireccioY,double nouDireccioZ);	//funcio per a canviar les dimensions de la esfera
  void CanviarDimensions(double nouDimensionsR, double nouDimensionsT);
  void Dibuixar();
  ~SeccioEsfera(){};
};

/*
class Mallat
{
protected:
  
  int NumVertex;
  int NumArestes;
  int NumCares;
  int NumTetra;
  
  int IncrementNumElements;
  int TamanyMallat;
  
  float* Vertex;
  float* Arestes;
  float* Cares;
  float* Tetra;
  float* Densitats;
  int* Materials;
  bool* Acompleix;
  
public:
  
  //Variables
  Mallat();
  void BorrarMalla();
  void Afig(float Vertex1[3], float Vertex2[3], float Vertex3[3], float Vertex4[3], int Material, float Densitats);
  void Elimina(int NumElement);
  int LlegirNumElements();
  void LlegirVertex(int NumElement, float V1[3], float V2[3], float V3[3], float V4[3]);
  int LlegirMaterial(int NumElement);
  float LlegirDensitat(int NumElement);
  void ModificarVertex(int NumElement, float Vertex1[3], float Vertex2[3], float Vertex3[3], float Vertex4[3]);
  void ModificarMaterial(int NumElement,int NouMaterial);
  void ModificarDensitat(int NumElement,int NouDensitat);
  ~Mallat(){};
};
*/
void RotarPunts(int NumPunts, float PuntsInterns[][3], double Alfa, double Beta, double Gamma);	//Funcio que gira un vector de punts tridimensionals
void escriure(std::string text,float x,float y, float z, float Tamany);    		//imprimira el text en pantalla
void escriure2(std::string text,float x,float y, float z, float Tamany);
template <typename tipus> void ordenar(tipus*,int);  //Ordena el vector de doubles que li pases de menor a major
template <typename tipus> void copiarVector(tipus* v1, tipus* v2, int NombreElements); //Copia v1 en v2

//--------------------------------Funcio ordenar, ordena el vector de doubles de menor a major----------------//
template <typename tipus> void ordenar(tipus* vector, int Npunts)
{
  int i=1;
  tipus auxvalor=0;

  do
  {
    if(vector[i-1]<=vector[i])
    {
      i=i+1;
    }
    else
    {
      auxvalor=vector[i-1];
      vector[i-1]=vector[i];
      vector[i]=auxvalor;
      i=i-1;
      if(i==0)
      {
	i=1;
      }
    }
  }while(i<Npunts);
  
}
//--------------------------------Funcio copiar vector, copia el vector del primer argument al punter del segon----------------//
template <typename tipus> void copiarVector(tipus* v1, tipus* v2, int NombreElements)
{
  int i=0;
  while(i<NombreElements)
    {
      v2[i] = v1[i];
      i++;
    }
}


#endif
